@@ -11,12 +11,12 @@ use tokio::net::UnixStream;
 use uuid::Uuid;
 
 use super::{
-    CharacteristicFlags, CharacteristicReader, CharacteristicWriter, DescriptorFlags, WriteValueType,
+    CharacteristicFlags, CharacteristicReader, CharacteristicWriter, DescriptorFlags, WriteOp,
     CHARACTERISTIC_INTERFACE, DESCRIPTOR_INTERFACE, SERVICE_INTERFACE,
 };
 use crate::{
-    all_dbus_objects, Address, Device, Error, ErrorKind, Event, Result, SessionInner, SingleSessionToken,
-    SERVICE_NAME, TIMEOUT,
+    all_dbus_objects, Address, Device, Error, ErrorKind, Event, InternalErrorKind, Result, SessionInner,
+    SingleSessionToken, SERVICE_NAME, TIMEOUT,
 };
 
 // ===========================================================================================
@@ -157,14 +157,19 @@ define_properties!(
         property(
             Uuid, Uuid,
             dbus: (SERVICE_INTERFACE, "UUID", String, MANDATORY),
-            get: (uuid, v => {v.parse().map_err(|_| Error::new(ErrorKind::InvalidUuid(v.to_string())))?}),
+            get: (uuid, v => {v.parse().map_err(|_| Error::new(ErrorKind::Internal(InternalErrorKind::InvalidUuid(v.to_string()))))?}),
         );
 
-        /// Service handle.
+        /// Service ids of included services of this service.
         property(
-            Handle, u16,
-            dbus: (SERVICE_INTERFACE, "Handle", u16, MANDATORY),
-            get: (handle, v => { v.to_owned() }),
+            Includes, Vec<u16>,
+            dbus: (SERVICE_INTERFACE, "Includes", Vec<Path>, MANDATORY),
+            get: (includes, v => {
+                v.into_iter().filter_map(|path| match Service::parse_dbus_path(path) {
+                    Some((_, _, service_id)) => Some(service_id),
+                    None => None
+                }).collect()
+            }),
         );
     }
 );
@@ -332,9 +337,9 @@ impl Characteristic {
     /// Acquire file descriptor and MTU for writing.
     ///
     /// It only works with characteristic that has
-    /// the `write_without_response` flag set.
+    /// the [write_without_response](CharacteristicFlags::write_without_response) flag set.
     ///
-    /// Usage of `write` will be
+    /// Usage of [write](Self::write) will be
     /// locked causing it to return NotPermitted error.
     /// To release the lock the client shall drop the writer.
     ///
@@ -397,16 +402,16 @@ impl Characteristic {
     /// Acquire file descriptor and MTU for notify.
     ///
     /// It only works with characteristic that has
-    /// the `notify` flag set and no other client has called
-    /// `StartNotify`.
+    /// the [notify](CharacteristicFlags::notify) flag set and no other client has called
+    /// [notify](Self::notify) or [notify_io](Self::notify_io).
     ///
     /// Notification are enabled during this procedure so
-    /// `notify` shall not be called, any notification
+    /// [notify](Self::notify) shall not be called, any notification
     /// will be dispatched via file descriptor therefore the
     /// Value property is not affected during the time where
     ///	notify has been acquired.
     ///
-    /// Usage of `noify` will be
+    /// Usage of [noify](Self::notify) will be
     /// locked causing it to return NotPermitted error.
     /// To release the lock the client shall drop the writer.
     ///
@@ -447,7 +452,7 @@ pub struct CharacteristicWriteRequest {
     /// Start offset.
     pub offset: u16,
     /// Write operation type.
-    pub op_type: WriteValueType,
+    pub op_type: WriteOp,
     /// True if prepare authorization request.
     pub prepare_authorize: bool,
 }
@@ -468,14 +473,7 @@ define_properties!(
         property(
             Uuid, Uuid,
             dbus: (CHARACTERISTIC_INTERFACE, "UUID", String, MANDATORY),
-            get: (uuid, v => {v.parse().map_err(|_| Error::new(ErrorKind::InvalidUuid(v.to_string())))?}),
-        );
-
-        /// Characteristic handle.
-        property(
-            Handle, u16,
-            dbus: (CHARACTERISTIC_INTERFACE, "Handle", u16, MANDATORY),
-            get: (handle, v => { v.to_owned() }),
+            get: (uuid, v => {v.parse().map_err(|_| Error::new(ErrorKind::Internal(InternalErrorKind::InvalidUuid(v.to_string()))))?}),
         );
 
         ///	True, if notifications or indications on this
@@ -686,14 +684,7 @@ define_properties!(
         property(
             Uuid, Uuid,
             dbus: (DESCRIPTOR_INTERFACE, "UUID", String, MANDATORY),
-            get: (uuid, v => {v.parse().map_err(|_| Error::new(ErrorKind::InvalidUuid(v.to_string())))?}),
-        );
-
-        /// Characteristic descriptor handle.
-        property(
-            Handle, u16,
-            dbus: (DESCRIPTOR_INTERFACE, "Handle", u16, MANDATORY),
-            get: (handle, v => { v.to_owned() }),
+            get: (uuid, v => {v.parse().map_err(|_| Error::new(ErrorKind::Internal(InternalErrorKind::InvalidUuid(v.to_string()))))?}),
         );
 
         /// Defines how the descriptor value can be used.
